function [Rho,epsilon] = ProjectedNewtonStep(Rho,epsilon,fC,sigma1,Meas,psiHat,phase1rho,eta,mMax)
% Computes the projected Newton's update
%
% Copyright 2015, Armin Eftekhari and Michael Wakin
%
% This file is part of the Greedy Super-Resolution Toolbox version 1.0.
%
%    The Greedy Super-Resolution Toolbox is free software: you can 
%    redistribute it and/or modify it under the terms of the GNU General 
%    Public License as published by the Free Software Foundation, either 
%    version 3 of the License, or (at your option) any later version.
%
%    The Greedy Super-Resolution Toolbox is distributed in the hope that it 
%    will be useful, but WITHOUT ANY WARRANTY; without even the implied 
%    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
%    See the GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with the Greedy Super-Resolution Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.
%

% F is the function we wish to minimize (locally)
e = @(rho,f) exp(-1i*2*pi*[-f:f]'*rho');
F = @(RhoIn) norm(Meas-(diag(psiHat)*e(RhoIn,fC))*pinv(diag(psiHat)*e(RhoIn,fC))*Meas)^2;


% Compute the gradient and Hessian at Rho
[Gradient,Hessian] = Gradient_Hessian(Rho,fC,Meas,psiHat);

% Calculate the active coordinates of Rho
Dist = dist(Rho,phase1rho); 
active = find((Dist >= sigma1-epsilon) .* (Dist <= sigma1));
if ~isempty(active); display('Active coordinate detected!'); end

% Calculate the reduced Hessian
aHessian = Hessian;
aHessian(active,:) = 0;
aHessian(:,active) = 0;
aHessian(active,active) = eye(length(active));
step = 1;
% Check PD
temp = min(eig(aHessian));
if temp <= 0
    display('Hessian not positive definite');
    aHessian = step*eye(length(Rho)); % Just use gradient descent
end

% Compute the descent direction
v = real(aHessian\Gradient);

% Exit?
brk = norm(dist(Proj(Rho-v,phase1rho,sigma1),Rho));
if  brk <= eta
    return 
else
    epsilon = min([brk,sigma1/2]);
end



% Line search
noDrop = 1;
m = 0;
lambda = 1;
while noDrop && (m < mMax)   
    rhoLambda = Proj(Rho-lambda*v,phase1rho,sigma1);
    
    if F(rhoLambda) - F(Rho) <= -.0001/lambda * (norm(dist(rhoLambda,Rho)))^2 % Has the objective reduced sufficiently?
        break
    end

    m = m+1;
    lambda = lambda/2;
end




% Update 
Rho = rhoLambda;

    

